/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: iaom <zhangpengfei@kylinos.cn>
 */

#ifndef UKUI_PANEL_PANEL_NEXT_H
#define UKUI_PANEL_PANEL_NEXT_H
#include <QTimer>
#include <QMenu>
#include <types.h>
#include <window-helper.h>
#include <island-view.h>
#include <widget-container.h>
#include "screens-manager.h"
#include "common.h"
#include "general-config-define.h"
#include "blur-region-helper.h"

namespace UkuiPanel {
class AutoHideHelper : public QObject
{
    Q_OBJECT

public:
    explicit AutoHideHelper(Screen *screen, QObject* parent = nullptr);
    void setScreen(Screen *screen);
    bool needAutoHide();

private:
    void update();
    Screen *m_screen = nullptr;
    bool m_needAutoHide = false;
    QString m_activeWindow;

Q_SIGNALS:
    void autoHideStateChanged(bool);
};
class PanelNext : public UkuiQuick::IslandView
{
    Q_OBJECT
    Q_PROPERTY(bool lockPanel READ lockPanel NOTIFY lockPanelChanged)
    Q_PROPERTY(bool enableCustomSize READ enableCustomSize NOTIFY enableCustomSizeChanged)

public:
    PanelNext(Screen* screen, const QString&id, QWindow* parent = nullptr);
    ~PanelNext();

    void setPanelScreen(Screen* screen);
    bool lockPanel() const;
    bool enableCustomSize() const;
    void addIsland(const QString &id);
    void removeIsland(const QString &id);
    void dragRemove(const QString &id);
    void dragAdding(const QString &id, bool addWhenDrop);

    Q_INVOKABLE void setPanelLength(int length);

Q_SIGNALS:
    void lockPanelChanged();
    void enableCustomSizeChanged();
    void islandRemoved(const QString &id);

protected:
    bool event(QEvent* e) override;

private Q_SLOTS:
    void onScreenGeometryChanged(const QRect&geometry);

    void setPanelPolicy(GeneralConfigDefine::PanelSizePolicy sizePolicy);

    void setPosition(UkuiQuick::Types::Pos position);

    void setPanelSize(int size);

    void setLockPanel(bool locked, bool noEmit = false);

    void setAutoHide(bool autoHide);

    void setHidden(bool hidden);

    void activeHideTimer(bool active = true);

    void onBlurRegionChanged(QRegion region);

    void changeForm(const QString &id, GeneralConfigDefine::IslandsForm form);

    void updateHideState();

private:
    void initMainContainer();

    void initIslandsConfig();

    void updateGeometry();

    void initConfig();

    void refreshRect();

    void updateMask();

    void initPanelConfig();

    void loadActions();

    void initSizeAction();

    void updateSizeAction();

    void initPositionAction();

    void updatePositionAction();

    void initIslandFormAction();

    void updateIslandFormAction();

    void disableWidget(const QString &id, bool disable);

    void updateDisableWidget();
private:
    QString m_id;
    UkuiQuick::WindowProxy* m_windowProxy = nullptr;
    Screen* m_screen{nullptr};
    int m_panelSize{56};
    UnavailableArea m_unavailableArea;
    GeneralConfigDefine::PanelSizePolicy m_sizePolicy{GeneralConfigDefine::Small};

    QAction* m_sizeAction{nullptr};
    QAction* m_lockPanelAction{nullptr};
    QAction* m_autoHideAction{nullptr};
    QAction* m_dataIslandFormAction{nullptr};
    QAction* m_settingsIslandFormAction{nullptr};

    bool m_isHidden = false;//当前是否已经隐藏
    bool m_autoHide = false;//是否自动隐藏(鼠标离开任务栏区域即隐藏)
    bool m_needHide = false;//当前是否有激活的最大化窗口需要任务栏隐藏
    bool m_lockPanel = false;
    bool m_startUp = true;
    QTimer* m_hideTimer = nullptr;
    QMap<QString, UkuiQuick::WidgetContainer *> m_islands;
    BlurRegionHelper* m_blurHelper = nullptr;
    QList<QAction *> m_dataIslandActions;
    QList<QAction *> m_appIslandActions;
    QList<QAction *> m_settingsIslandActions;
    QList<QMenu *> m_menus; //用于QMenu的统一销毁
    AutoHideHelper *m_autoHideHelper = nullptr;
    bool m_containsMouse = false;

    QStringList m_disabledWidgets;
};
} // UkuiPanel

#endif //UKUI_PANEL_PANEL_NEXT_H
